<?php

namespace Drupal\splidebox;

use Drupal\Core\Url;
use Drupal\blazy\BlazyEntityInterface;
// @todo enable post Blazy:2.10.
// use Drupal\blazy\Field\BlazyField;
use Drupal\splide\Form\SplideAdminInterface;
use Drupal\splide\Entity\Splide;
use Drupal\splide\SplideManager;

/**
 * Adds Splidebox functionality.
 */
class Splidebox extends SplideManager implements SplideboxInterface {

  /**
   * The splide admin service.
   *
   * @var \Drupal\splide\Form\SplideAdminInterface
   */
  protected $admin;

  /**
   * The blazy entity service.
   *
   * @var \Drupal\blazy\BlazyEntityInterface
   */
  protected $blazyEntity;

  /**
   * The lightbox dummy template.
   *
   * @var array
   */
  private $template;

  /**
   * Returns the admin service.
   */
  public function setAdmin(SplideAdminInterface $admin) {
    $this->admin = $admin;
    return $this;
  }

  /**
   * Returns the blazy entity service.
   */
  public function setBlazyEntity(BlazyEntityInterface $blazy_entity) {
    $this->blazyEntity = $blazy_entity;
    return $this;
  }

  /**
   * Implements hook_blazy_attach_alter().
   */
  public function attachAlter(array &$load, array $attach = []) {
    $data = $this->toData($load, $attach);

    $this->moduleHandler->alter('splidebox_attach', $data);

    $load['drupalSettings']['splidebox'] = [
      'lightbox' => $data['lightbox'],
      'splide'   => $data['splide'],
      'template' => $data['template'],
    ];

    $load['library'][] = 'splidebox/load';
    if (!empty($attach['ajax_link'])) {
      $load['library'][] = 'splidebox/ajax';
    }

    $this->skinManager->attachCore($load, $data['attach']);
  }

  /**
   * Overrides variables for theme_blazy().
   */
  public function preprocessBlazy(array &$variables) {
    $settings = &$variables['settings'];

    // Provides AJAX contents if so configured.
    if (!empty($settings['ajax_link'])) {
      $this->toUrl($variables);
    }
  }

  /**
   * Implements hook_blazy_complete_form_element_alter().
   */
  public function formElementAlter(array &$form, array $definition = []) {
    $namespace   = $definition['namespace'] ?? '';
    $namespaces  = ['blazy', 'splide', 'gridstack'];
    $field_type  = $definition['field_type'] ?? '';
    $target_type = $definition['target_type'] ?? '';
    $bundles     = $definition['target_bundles'] ?? [];
    $settings    = $definition['settings'] ?? [];
    $links       = ['link', 'string'];
    $entities    = ['entity_reference', 'entity_reference_revisions'];
    $supported   = $namespace && in_array($namespace, $namespaces);
    $applicable  = $supported && ($field_type && in_array($field_type, $entities));

    if ($applicable && isset($form['media_switch'])) {
      $options = $this->admin->getFieldOptions($bundles, $links, $target_type);
      $form['ajax_link'] = [
        '#type'          => 'select',
        '#title'         => $this->t('Lightbox AJAX link'),
        '#options'       => $options,
        '#empty_option'  => $this->t('- None -'),
        '#default_value' => $settings['ajax_link'] ?? '',
        '#weight'        => -98.99,
        '#states'        => [
          'visible' => [
            'select[name*="[media_switch]"]' => ['value' => 'splidebox'],
          ],
        ],
        '#description'   => $this->t('Choose any single-value link field types which have urls (<code>/node/123</code>) to nodes (other entities or forms are not supported to avoid complications). It will be loaded as the lightbox AJAX contents. Specific for Media, be sure the same link field available for all media types to have a mixed of Media correctly. To replace images/ videos with AJAX contents only, override <code>ajaxOnly</code> option to TRUE via <code>hook_splidebox_attach_alter</code>. Default is FALSE, the non-zoomable lightbox image and video are embedded above AJAX contents as hybrid contents.'),
      ];
    }
  }

  /**
   * Overrides variables for theme_blazy().
   */
  private function toUrl(array &$variables) {
    $settings = $variables['settings'];
    $blazies  = $settings['blazies'] ?? NULL;
    $langcode = $settings['langcode'] ?? 'en';
    $langcode = $blazies ? $blazies->get('language.current', $langcode) : $langcode;
    $box_url  = $settings['box_url'] ?? '';
    $box_url  = $blazies ? $blazies->get('lightbox.url', $box_url) : $box_url;

    // Bail out if no image item is provided.
    // @todo make it flexible.
    if (empty($variables['item'])) {
      return;
    }

    $item = $variables['item'];
    $entity = $item->getEntity() ?: $item->entity;

    if ($entity && !$entity->isNew()) {
      $link = $settings['ajax_link'];
      // @todo enable post Blazy:2.10.
      // $value = BlazyField::getString($entity, $link, $langcode, TRUE);
      $value = $this->blazyEntity->getFieldString($entity, $link, $langcode, TRUE);
      if (empty($value)) {
        return;
      }

      if (mb_strpos($value, 'internal:') !== FALSE
        || mb_strpos($value, 'entity:') !== FALSE
        || mb_strpos($value, 'route:') !== FALSE) {
        $url = Url::fromUri($value);
      }
      else {
        $url = Url::fromUserInput($value);
      }

      if ($path = $url->setAbsolute()->toString()) {
        $old_url = empty($variables['url']) ? '' : $variables['url'];
        $variables['url'] = $path;
        $variables['url_attributes']['data-b-ajax'] = $value;
        $variables['url_attributes']['data-box-url'] = empty($box_url) ? $old_url : $box_url;
      }
    }
  }

  /**
   * Returns splidebox data.
   */
  private function toData(array &$load, array $attach) {
    $name      = $attach['box_optionset'] ?? NULL;
    $name      = $name ?: 'splidebox';
    $optionset = Splide::loadWithFallback($name);
    $options   = $optionset->getSettings();
    $layout    = $attach['box_layout'] ?? NULL;
    $pos       = $attach['box_caption_pos'] ?? NULL;

    $this->modifyOptions($options, $attach, $optionset);

    // box_caption_pos: inline (ala Colorbox), or overlay (ala PhotoSwipe).
    // box_layout: any usual caption layouts, requires theming except bottom.
    // To replace media with AJAX contents only, override ajaxOnly to TRUE.
    // Play with longer ajaxTimeout if failing due to heavy AJAX contents.
    $lightbox = [
      'skin'        => $attach['skin'] ?? 'default',
      'layout'      => $layout ?: 'bottom',
      'captionPos'  => $pos ?: 'overlay',
      'ajaxOnly'    => FALSE,
      'ajaxTimeout' => 600,
    ];

    $settings = $lightbox;
    $settings['blazies'] = $attach['blazies'] ?? NULL;

    $build = [
      'options'   => $options,
      'optionset' => $optionset,
      'settings'  => $settings,
    ];

    return [
      'lightbox' => $lightbox,
      'splide'   => Splide::typecast($options, FALSE),
      'load'     => $load,
      'attach'   => $attach,
      'template' => $this->toTemplate($build),
    ];
  }

  /**
   * Modifies the splidebox options.
   */
  private function modifyOptions(array &$options, array &$attach, Splide $optionset) {
    $blazies  = $attach['blazies'] ?? NULL;
    $excludes = ['classes', 'i18n', 'padding'];
    $excludes = array_combine($excludes, $excludes);
    $options  = array_diff_key($options, $excludes);
    $skin     = $optionset->getSkin() ?: 'default';
    $video    = empty($options['video']) ? [] : (array) $options['video'];
    $autoplay = $video['autoplay'] ?? TRUE;

    // Some are hidden module features. Wheel is inherent with Zoom, disable.
    $attach['skin'] = empty($attach['skin_lightbox']) ? $skin : $attach['skin_lightbox'];
    $attach['media'] = $attach['zoom'] = TRUE;
    $attach['fullscreen'] = $options['fullscreen'] = TRUE;
    $options['video'] = $video;
    $options['video']['autoplay'] = $autoplay;
    $options['wheel'] = FALSE;

    if ($blazies && $count = $blazies->get('count')) {
      $options['count'] = $count;
    }

    $zoom = [
      'on'          => TRUE,
      'click'       => TRUE,
      'scale'       => TRUE,
      'min'         => 1,
      'max'         => 1.5,
      'root'        => '.splidebox',
      'rootClass'   => 'is-sbox-zoomed',
      'dragClass'   => 'is-sbox-dragging',
      'target'      => '.splidebox__item:not(.is-ajax-item)',
      'targetClass' => 'is-sbox-moved',
    ];
    $zooms = empty($options['zoom']) ? [] : (array) $options['zoom'];
    $options['zoom'] = array_merge($zoom, $zooms);
  }

  /**
   * Returns a dummy lightbox template.
   */
  private function toTemplate(array $build) {
    if (!isset($this->template)) {
      $items = [];
      foreach (['One', 'Two'] as $key) {
        $items[] = [
          'slide' => ['#markup' => $key],
        ];
      }

      $build['items'] = $items;
      $html = $this->build($build);
      $this->template = $this->renderer->renderPlain($html);
    }
    return $this->template;
  }

}
